@@ -136,7 +136,7 @@ For a full list of contributors, please check <a href="https://github.com/indili
 /* INDI Library version */
 #define INDI_VERSION_MAJOR   2
 #define INDI_VERSION_MINOR   0
-#define INDI_VERSION_RELEASE 4
+#define INDI_VERSION_RELEASE 5
 
 /*******************************************************************************
  * Manifest constants
@@ -510,11 +510,6 @@ extern "C" {
 #endif
 
 // FIXME: duplicated from indidevapi.h. Can we share ?
-
-// Advertize support for shared blob on this platform
-#define INDI_SHARED_BLOB_SUPPORT
-#include "sharedblob.h"
-
 #ifdef __cplusplus
 }
 #endif
