@@ -19,7 +19,8 @@
 #pragma once
 
 #include "defaultdevice.h"
-
+#include "libastro.h"
+#include "indipropertyswitch.h"
 #include <libnova/julian_day.h>
 
 #include <string>
@@ -56,7 +57,7 @@
  *              changes the custom tracking rates while the mount is tracking, it it sent to the child class via SetTrackRate(...) function.
  *              The base class will reject any track rates that switch from positive to negative (reverse) tracking rates as the mount must be stopped before
  *              such change takes place.
- * + TrackState: Engages or Disengages tracking. When engaging tracking, the child class should take the necessary steps to set the appropiate TrackMode and TrackRate
+ * + TrackState: Engages or Disengages tracking. When engaging tracking, the child class should take the necessary steps to set the appropriate TrackMode and TrackRate
  *               properties before or after engaging tracking as governed by the mount protocol.
  *
  * Ideally, the child class should avoid changing property states directly within a function call from the base class as such state changes take place in the base class
@@ -105,12 +106,13 @@ class Telescope : public DefaultDevice
         };
         enum TelescopeParkData
         {
-            PARK_NONE,
-            PARK_RA_DEC,
-            PARK_HA_DEC,
-            PARK_AZ_ALT,
-            PARK_RA_DEC_ENCODER,
-            PARK_AZ_ALT_ENCODER
+            PARK_NONE,              /*!< Mount does not support any form of parking */
+            PARK_RA_DEC,            /*!< Park to specific RA/DE coordinate. Deprecated, do not use */
+            PARK_HA_DEC,            /*!< Park to specific HA/DE coordinate. Hour Angle & Declination degrees */
+            PARK_AZ_ALT,            /*!< Park to specific AZ/ALT coordinate. Azimuth & Altitude degrees */
+            PARK_RA_DEC_ENCODER,    /*!< Park to specific HA/DE encoder. Hour Angle & Declination steps */
+            PARK_AZ_ALT_ENCODER,    /*!< Park to specific AZ/ALT encoder. Azimuth & Altitude steps */
+            PARK_SIMPLE             /*!< Only Park or Unpark is known but location is unknown. */
         };
         enum TelescopeLocation
         {
@@ -169,6 +171,7 @@ class Telescope : public DefaultDevice
             TELESCOPE_HAS_TRACK_RATE              = 1 << 10, /** Does the telescope have custom track rates? */
             TELESCOPE_HAS_PIER_SIDE_SIMULATION    = 1 << 11, /** Does the telescope simulate the pier side property? */
             TELESCOPE_CAN_TRACK_SATELLITE         = 1 << 12, /** Can the telescope track satellites? */
+            TELESCOPE_CAN_FLIP                    = 1 << 13, /** Does the telescope have a command for flipping? */
         } TelescopeCapability;
 
         Telescope();
@@ -191,12 +194,12 @@ class Telescope : public DefaultDevice
         /**
          * @brief SetTelescopeCapability sets the Telescope capabilities. All capabilities must be initialized.
          * @param cap ORed list of telescope capabilities.
-         * @param slewRateCount Number of slew rates supported by the telescope. If < 4 (default is 0),
+         * @param slewRateCount Number of slew rates supported by the telescope. If < 4,
          * no slew rate properties will be defined to the client. If >=4, the driver will construct the default
          * slew rate property TELESCOPE_SLEW_RATE with SLEW_GUIDE, SLEW_CENTERING, SLEW_FIND, and SLEW_MAX
          * members where SLEW_GUIDE is the at the lowest setting and SLEW_MAX is at the highest.
          */
-        void SetTelescopeCapability(uint32_t cap, uint8_t slewRateCount = 0);
+        void SetTelescopeCapability(uint32_t cap, uint8_t slewRateCount);
 
         /**
          * @return True if telescope support goto operations
@@ -214,6 +217,14 @@ class Telescope : public DefaultDevice
             return capability & TELESCOPE_CAN_SYNC;
         }
 
+        /**
+         * @return True if telescope support sync operations
+         */
+        bool CanFlip()
+        {
+            return capability & TELESCOPE_CAN_FLIP;
+        }
+
         /**
          * @return True if telescope can abort motion.
          */
@@ -340,7 +351,7 @@ class Telescope : public DefaultDevice
          * ~/.indi/ParkData.xml) is updated in the process.
          * @param isparked set to true if parked, false otherwise.
          */
-        void SetParked(bool isparked);
+        virtual void SetParked(bool isparked);
 
         /**
          * @return Get current RA/AZ parking position.
@@ -449,11 +460,19 @@ class Telescope : public DefaultDevice
         virtual bool ReadScopeStatus() = 0;
 
         /**
-         * \brief Move the scope to the supplied RA and DEC coordinates
-         * \return True if successful, false otherwise
-         * \note If not implemented by the child class, this function by default returns false with a
-         * warning message.
-         */
+        * \brief Move and flip the scope to the supplied RA and DEC coordinates
+        * \return True if successful, false otherwise
+        * \note If not implemented by the child class, this function by default returns false with a
+        * warning message.
+        */
+        virtual bool Flip(double ra, double dec);
+
+        /**
+        * \brief Move the scope to the supplied RA and DEC coordinates
+        * \return True if successful, false otherwise
+        * \note If not implemented by the child class, this function by default returns false with a
+        * warning message.
+        */
         virtual bool Goto(double ra, double dec);
 
         /**
@@ -606,6 +625,13 @@ class Telescope : public DefaultDevice
          */
         virtual bool SetDefaultPark();
 
+
+        /**
+         * @brief SyncParkStatus Update the state and switches for parking
+         * @param isparked True if parked, false otherwise.
+         */
+        virtual void SyncParkStatus(bool isparked);
+
         /**
          * @brief SetSlewRate Set desired slew rate index.
          * @param index Index of slew rate where 0 is slowest rate and capability.nSlewRate-1 is maximum rate.
@@ -637,27 +663,8 @@ class Telescope : public DefaultDevice
          */
         TelescopePierSide expectedPierSide(double ra);
 
-        // helper functions
-        double getAzimuth(double r, double d);
-
-        ln_lnlat_posn lnobserver { 0, 0 };
-
-        /**
-         * @brief Load scope settings from XML files.
-         * @return True if all config values were loaded otherwise false.
-         */
-        bool LoadScopeConfig();
-
-        /**
-         * @brief Load scope settings from XML files.
-         * @return True if Config #1 exists otherwise false.
-         */
-        bool HasDefaultScopeConfig();
-
-        /**
-         * \brief Save scope settings to XML files.
-         */
-        bool UpdateScopeConfig();
+        // Geographic Location
+        IGeographicCoordinates m_Location { 0, 0, 0 };
 
         /**
          * @brief Validate a file name
@@ -707,7 +714,7 @@ class Telescope : public DefaultDevice
 
         // On a coord_set message, sync, or slew
         ISwitchVectorProperty CoordSP;
-        ISwitch CoordS[3];
+        ISwitch CoordS[4];
 
         // A number vector that stores lattitude and longitude
         INumberVectorProperty LocationNP;
@@ -740,14 +747,18 @@ class Telescope : public DefaultDevice
         ISwitch MovementWES[2];
         ISwitchVectorProperty MovementWESP;
 
+        // Reverse NS or WE
+        INDI::PropertySwitch ReverseMovementSP {2};
+        enum
+        {
+            REVERSE_NS,
+            REVERSE_WE
+        };
+
         // Slew Rate
         ISwitchVectorProperty SlewRateSP;
         ISwitch *SlewRateS {nullptr};
 
-        // Telescope & guider aperture and focal length
-        INumber ScopeParametersN[4];
-        INumberVectorProperty ScopeParametersNP;
-
         // UTC and UTC Offset
         IText TimeT[2] {};
         ITextVectorProperty TimeTP;
@@ -828,7 +839,7 @@ class Telescope : public DefaultDevice
          */
         ISwitchVectorProperty TrackSatSP;
         ISwitch TrackSatS[SAT_TRACK_COUNT];
-        
+
         // PEC State
         ISwitch PECStateS[2];
         ISwitchVectorProperty PECStateSP;
@@ -860,45 +871,14 @@ class Telescope : public DefaultDevice
         Connection::Serial *serialConnection = nullptr;
         Connection::TCP *tcpConnection       = nullptr;
 
-        // XML node names for scope config
-        const std::string ScopeConfigRootXmlNode { "scopeconfig" };
-        const std::string ScopeConfigDeviceXmlNode { "device" };
-        const std::string ScopeConfigNameXmlNode { "name" };
-        const std::string ScopeConfigScopeFocXmlNode { "scopefoc" };
-        const std::string ScopeConfigScopeApXmlNode { "scopeap" };
-        const std::string ScopeConfigGScopeFocXmlNode { "gscopefoc" };
-        const std::string ScopeConfigGScopeApXmlNode { "gscopeap" };
-        const std::string ScopeConfigLabelApXmlNode { "label" };
-
-        // A switch to apply custom aperture/focal length config
-        enum
-        {
-            SCOPE_CONFIG1,
-            SCOPE_CONFIG2,
-            SCOPE_CONFIG3,
-            SCOPE_CONFIG4,
-            SCOPE_CONFIG5,
-            SCOPE_CONFIG6
-        };
-        ISwitch ScopeConfigs[6];
-        ISwitchVectorProperty ScopeConfigsSP;
-
-        // Scope config name
-        ITextVectorProperty ScopeConfigNameTP;
-        IText ScopeConfigNameT[1] {};
-
-        /// The telescope/guide scope configuration file name
-        const std::string ScopeConfigFileName;
+        bool IsParked {false};
+        TelescopeParkData parkDataType {PARK_NONE};
 
     private:
         bool processTimeInfo(const char *utc, const char *offset);
         bool processLocationInfo(double latitude, double longitude, double elevation);
         void triggerSnoop(const char *driverName, const char *propertyName);
-        /**
-         * @brief SyncParkStatus Update the state and switches for parking
-         * @param isparked True if parked, false otherwise.
-         */
-        void SyncParkStatus(bool isparked);
+        void generateCoordSet();
 
         /**
          * @brief LoadParkXML Read and process park XML data.
@@ -906,9 +886,7 @@ class Telescope : public DefaultDevice
          */
         const char *LoadParkXML();
 
-        TelescopeParkData parkDataType {PARK_NONE};
         bool IsLocked {true};
-        bool IsParked {false};
         const char *ParkDeviceName {nullptr};
         const std::string ParkDataFileName;
         XMLEle *ParkdataXmlRoot {nullptr}, *ParkdeviceXml {nullptr}, *ParkstatusXml {nullptr}, *ParkpositionXml {nullptr},
@@ -930,6 +908,9 @@ class Telescope : public DefaultDevice
         float motionDirWEValue {0};
 
         bool m_simulatePierSide;    // use setSimulatePierSide and getSimulatePierSide for public access
+
+        // 100 millisecond of arc or time.
+        static constexpr double EQ_NOTIFY_THRESHOLD {1.0 / (60 * 60 * 10)};
 };
 
 }
